/* 
 * Copyright 2014 SATO taichi
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package example;

import java.io.BufferedReader;
import java.nio.file.*;

import rx.Observable.OnSubscribe;
import rx.schedulers.Schedulers;

public class AsyncFileIO {
	public static void main(String[] args) {
		rx.Observable.create(make("build.gradle"))
			.subscribeOn(Schedulers.io())
			.observeOn(Schedulers.computation()).take(3)
			.map(AsyncFileIO::addThreadName).toBlockingObservable()
			.forEach(System.out::println);
	}

	static String addThreadName(String s) {
		return Thread.currentThread().getName() + " " + s;
	}

	static OnSubscribe<String> make(String path) {
		return (sub) -> {
			try (BufferedReader reader = Files.newBufferedReader(Paths
					.get(path))) {
				while (sub.isUnsubscribed() == false) {
					String line = reader.readLine();
					if (line == null) {
						break;
					}
					emit(sub, () -> {
						sub.onNext(addThreadName(line));
					});
				}
				emit(sub, () -> {
					sub.onCompleted();
				});
			} catch (Throwable t) {
				emit(sub, () -> {
					sub.onError(t);
				});
			}
		};
	}

	static <T> void emit(rx.Subscriber<T> sub, rx.functions.Action0 a) {
		if (sub.isUnsubscribed() == false) {
			a.call();
		}
	}
}
